from RG.xlsread import *
from RG.regress import *
from RG.R2 import *

from RG.PCA import *
from RG.PLS import *
from RG.mbPLS import *
from RG.LDA import *


from RG.colorspectra import *

from RG.center import *
from RG.autoscale import *
from RG.baseline import *
from RG.SNV import *
from RG.MSC import *
from RG.EMSC import *
from RG.savitzky_golay import *
from RG.detrend import *










